@@ -16,7 +16,7 @@ import cmd
 from datetime import datetime
 import getpass
 import locale
-import os
+import os.path
 import pkg_resources
 import shlex
 import shutil
@@ -32,7 +32,7 @@ from trac.core import TracError
 from trac.env import Environment
 from trac.perm import PermissionSystem
 from trac.ticket.model import *
-from trac.util import getuser
+from trac.util import WindowsError, getuser
 from trac.util.datefmt import parse_date, format_date, format_datetime, utc
 from trac.util.html import html
 from trac.util.text import to_unicode, wrap, unicode_quote, unicode_unquote, \
@@ -50,7 +50,12 @@ def printout(*args):
 def printerr(*args):
     console_print(sys.stderr, *args)
 
-def copytree(src, dst, symlinks=False, skip=[]):
+def makedirs(path, overwrite=False):
+    if overwrite and os.path.exists(path):
+        return
+    os.makedirs(path)
+
+def copytree(src, dst, symlinks=False, skip=[], overwrite=True):
     """Recursively copy a directory tree using copy2() (from shutil.copytree.)
 
     Added a `skip` parameter consisting of absolute paths
@@ -64,7 +69,7 @@ def copytree(src, dst, symlinks=False, skip=[]):
     skip = [str_path(f) for f in skip]
     def copytree_rec(src, dst):
         names = os.listdir(src)
-        os.mkdir(dst)
+        makedirs(dst, overwrite=overwrite)
         errors = []
         for name in names:
             srcname = os.path.join(src, name)
@@ -1154,7 +1159,7 @@ Congratulations!
             self.__env.upgrade(backup=do_backup)
         except TracError, e:
             msg = unicode(e)
-            if 'backup' in msg:
+            if 'backup' in msg.lower():
                 raise TracError("Backup failed '%s'.\nUse `--no-backup' to "
                                 "upgrade without doing a backup." % msg)
             else:
@@ -1243,7 +1248,7 @@ Congratulations!
                 printout('   ', source)
                 if os.path.exists(source):
                     dest = os.path.join(chrome_target, key)
-                    copytree(source, dest)
+                    copytree(source, dest, overwrite=True)
         
         # Create and copy scripts
         os.makedirs(script_target)
@@ -1253,9 +1258,11 @@ Congratulations!
             dest = os.path.join(script_target, 'trac.'+script)
             template = Chrome(env).load_template('deploy_trac.'+script, 'text')
             stream = template.generate(**data)
-            out = open(dest, 'w')
-            stream.render('text', out=out)
-            out.close()
+            out = os.fdopen(os.open(dest, os.O_CREAT | os.O_WRONLY), 'w')
+            try:
+                stream.render('text', out=out)
+            finally:
+                out.close()
 
 
 class TracAdminHelpMacro(WikiMacroBase):
