@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2008 Edgewall Software
+# Copyright (C) 2005-2009 Edgewall Software
 # Copyright (C) 2005 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
@@ -30,11 +30,12 @@ from trac.core import *
 from trac.perm import PermissionSystem, IPermissionRequestor
 from trac.util import get_pkginfo, get_module_path
 from trac.util.compat import partial
-from trac.util.text import to_unicode
+from trac.util.text import exception_to_unicode, to_unicode
 from trac.util.translation import _
 from trac.web import HTTPNotFound, IRequestHandler
-from trac.web.chrome import add_script, add_stylesheet, add_warning, Chrome, \
-                            INavigationContributor, ITemplateProvider
+from trac.web.chrome import add_notice, add_script, add_stylesheet, \
+                            add_warning, Chrome, INavigationContributor, \
+                            ITemplateProvider
 
 try:
     from webadmin import IAdminPageProvider
@@ -69,7 +70,7 @@ class AdminModule(Component):
     # IRequestHandler methods
 
     def match_request(self, req):
-        match = re.match('/admin(?:/([^/]+))?(?:/([^/]+))?(?:/(.*)$)?',
+        match = re.match('/admin(?:/([^/]+)(?:/([^/]+)(?:/(.+))?)?)?$',
                          req.path_info)
         if match:
             req.args['cat_id'] = match.group(1)
@@ -98,7 +99,11 @@ class AdminModule(Component):
         panel_id = req.args.get('panel_id')
         path_info = req.args.get('path_info')
         if not panel_id:
-            panel_id = filter(lambda panel: panel[0] == cat_id, panels)[0][2]
+            try:
+                panel_id = filter(
+                            lambda panel: panel[0] == cat_id, panels)[0][2]
+            except IndexError:
+                raise HTTPNotFound(_('Unknown administration panel'))
 
         provider = providers.get((cat_id, panel_id), None)
         if not provider:
@@ -169,6 +174,20 @@ class AdminModule(Component):
         return panels, providers
 
 
+def _save_config(config, req, log):
+    """Try to save the config, and display either a success notice or a
+    failure warning.
+    """
+    try:
+        config.save()
+        add_notice(req, _('Your changes have been saved.'))
+    except Exception, e:
+        log.error('Error writing to trac.ini: %s', exception_to_unicode(e))
+        add_warning(req, _('Error writing to trac.ini, make sure it is '
+                           'writable by the web server. Your changes have '
+                           'not been saved.'))
+
+
 class BasicsAdminPanel(Component):
 
     implements(IAdminPanelProvider)
@@ -185,7 +204,7 @@ class BasicsAdminPanel(Component):
         if req.method == 'POST':
             for option in ('name', 'url', 'descr'):
                 self.config.set('project', option, req.args.get(option))
-            self.config.save()
+            _save_config(self.config, req, self.log)
             req.redirect(req.href.admin(cat, page))
 
         data = {
@@ -213,7 +232,7 @@ class LoggingAdminPanel(Component):
         log_dir = os.path.join(self.env.path, 'log')
 
         log_types = [
-            dict(name='', label=_('None'), selected=False, disabled=False),
+            dict(name='none', label=_('None'), selected=log_type == 'none', disabled=False),
             dict(name='stderr', label=_('Console'),
                  selected=log_type == 'stderr', disabled=False),
             dict(name='file', label=_('File'), selected=log_type == 'file',
@@ -231,38 +250,37 @@ class LoggingAdminPanel(Component):
             changed = False
 
             new_type = req.args.get('log_type')
-            if new_type and new_type not in ('stderr', 'file', 'syslog',
-                                             'eventlog'):
+            if new_type not in [t['name'] for t in log_types]:
                 raise TracError(
                     _('Unknown log type %(type)s', type=new_type),
                     _('Invalid log type')
                 )
             if new_type != log_type:
-                self.config.set('logging', 'log_type', new_type or 'none')
+                self.config.set('logging', 'log_type', new_type)
                 changed = True
                 log_type = new_type
 
-            if log_type:
+            if log_type == 'none':
+                self.config.remove('logging', 'log_level')
+                changed = True
+            else:
                 new_level = req.args.get('log_level')
-                if new_level and new_level not in log_levels:
+                if new_level not in log_levels:
                     raise TracError(
                         _('Unknown log level %(level)s', level=new_level),
                         _('Invalid log level'))
-                if new_level and new_level != log_level:
+                if new_level != log_level:
                     self.config.set('logging', 'log_level', new_level)
                     changed = True
-                    log_evel = new_level
-            else:
-                self.config.remove('logging', 'log_level')
-                changed = True
+                    log_level = new_level
 
             if log_type == 'file':
                 new_file = req.args.get('log_file', 'trac.log')
                 if new_file != log_file:
-                    self.config.set('logging', 'log_file', new_file or '')
+                    self.config.set('logging', 'log_file', new_file)
                     changed = True
                     log_file = new_file
-                if log_type == 'file' and not log_file:
+                if not log_file:
                     raise TracError(_('You must specify a log file'),
                                     _('Missing field'))
             else:
@@ -270,7 +288,7 @@ class LoggingAdminPanel(Component):
                 changed = True
 
             if changed:
-                self.config.save()
+                _save_config(self.config, req, self.log),
             req.redirect(req.href.admin(cat, page))
 
         data = {
@@ -298,44 +316,55 @@ class PermissionAdminPanel(Component):
     def render_admin_panel(self, req, cat, page, path_info):
         perm = PermissionSystem(self.env)
         all_permissions = perm.get_all_permissions()
+        all_actions = perm.get_actions()
 
         if req.method == 'POST':
             subject = req.args.get('subject', '')
             action = req.args.get('action')
             group = req.args.get('group', '')
 
-            if subject and subject == subject.upper() or \
-                   group and group == group.upper():
+            if subject and subject.isupper() or \
+                   group and group.isupper():
                 raise TracError(_('All upper-cased tokens are reserved for '
                                   'permission names'))
 
             # Grant permission to subject
             if req.args.get('add') and subject and action:
                 req.perm.require('PERMISSION_GRANT')
-                if action not in perm.get_actions():
+                if action not in all_actions:
                     raise TracError(_('Unknown action'))
                 req.perm.require(action)
                 if (subject, action) not in all_permissions:
                     perm.grant_permission(subject, action)
+                    add_notice(req, _('The user %(subject)s has been granted '
+                                      'the permission %(action)s.',
+                                      subject=subject, action=action))
                     req.redirect(req.href.admin(cat, page))
                 else:
-                    add_warning(req,
-                            _('Permission "%(action)s" was already granted '
-                            'to "%(subject)s"', action=action,
-                            subject=subject))
+                    add_warning(req, _('The permission %(action)s was already '
+                                       'granted to %(subject)s.',
+                                       action=action, subject=subject))
 
             # Add subject to group
             elif req.args.get('add') and subject and group:
                 req.perm.require('PERMISSION_GRANT')
                 for action in perm.get_user_permissions(group):
-                    req.perm.require(action)
-                if (subject,group) not in all_permissions:
+                    if not action in all_actions: # plugin disabled?
+                        self.env.log.warn("Adding %s to group %s: " \
+                            "Permission %s unavailable, skipping perm check." \
+                            % (subject, group, action))
+                    else:
+                        req.perm.require(action)
+                if (subject, group) not in all_permissions:
                     perm.grant_permission(subject, group)
+                    add_notice(req, _('The user %(subject)s has been added to '
+                                      'the group %(group)s.', subject=subject,
+                                      group=group))
                     req.redirect(req.href.admin(cat, page))
                 else:
-                    add_warning(req, 
-                                _('"%(subject)s" was already added to group '
-                                '"%(group)s"', subject=subject, group=group))
+                    add_warning(req, _('The user %(subject)s was already '
+                                       'added to the group %(group)s.',
+                                       subject=subject, group=group))
 
             # Remove permissions action
             elif req.args.get('remove') and req.args.get('sel'):
@@ -346,10 +375,12 @@ class PermissionAdminPanel(Component):
                     subject, action = key.split(':', 1)
                     if (subject, action) in perm.get_all_permissions():
                         perm.revoke_permission(subject, action)
+                add_notice(req, _('The selected permissions have been '
+                                  'revoked.'))
                 req.redirect(req.href.admin(cat, page))
 
         return 'admin_perms.html', {
-            'actions': perm.get_actions(),
+            'actions': all_actions,
             'perms': all_permissions
         }
 
@@ -419,7 +450,7 @@ class PluginAdminPanel(Component):
         except AttributeError:
             # OS_BINARY not available on every platform
             pass
-        target_file = os.fdopen(os.open(target_path, flags), 'w')
+        target_file = os.fdopen(os.open(target_path, flags, 0666), 'w')
         try:
             shutil.copyfileobj(upload.file, target_file)
             self.log.info('Plugin %s installed to %s', plugin_filename,
@@ -465,7 +496,7 @@ class PluginAdminPanel(Component):
                 changes = True
 
         if changes:
-            self.config.save()
+            _save_config(self.config, req, self.log)
 
     def _render_view(self, req):
         plugins = {}
@@ -501,7 +532,21 @@ class PluginAdminPanel(Component):
                             if k == 'home_page' or k == 'url':
                                 k = 'home_page'
                                 v = v.replace('$', '').replace('URL: ', '') 
+                            if k == 'author':
+                                v = to_unicode(v)
                             info[k] = v
+                else:
+                    # Info found; set all those fields to "None" that have the 
+                    # value "UNKNOWN" as this is the value for fields that
+                    # aren't specified in "setup.py"
+                    for k in info:
+                        if info[k] == 'UNKNOWN':
+                            info[k] = None
+                        elif k == 'author':
+                            # Must be encoded as unicode as otherwise Genshi 
+                            # may raise a "UnicodeDecodeError".
+                            info[k] = to_unicode(info[k])
+
                 # retrieve plugin version info
                 version = dist.version
                 if not version:
