@@ -28,7 +28,7 @@ from trac.config import *
 from trac.core import Component, ComponentManager, implements, Interface, \
                       ExtensionPoint, TracError
 from trac.db import DatabaseManager
-from trac.util import get_pkginfo
+from trac.util import create_file, get_pkginfo
 from trac.util.text import exception_to_unicode
 from trac.util.translation import _
 from trac.versioncontrol import RepositoryManager
@@ -164,13 +164,13 @@ class Environment(Component, ComponentManager):
          - $(basename)s the last path component of the current environment
          - $(project)s  the project name
 
-         Note the usage of `$(...)s` instead of `%(...)s` as the latter form
-         would be interpreted by the ConfigParser itself.
+        Note the usage of `$(...)s` instead of `%(...)s` as the latter form
+        would be interpreted by the ConfigParser itself.
 
-         Example:
-         ($(thread)d) Trac[$(basename)s:$(module)s] $(levelname)s: $(message)s
+        Example:
+        `($(thread)d) Trac[$(basename)s:$(module)s] $(levelname)s: $(message)s`
 
-         (since 0.10.5)""")
+        ''(since 0.10.5)''""")
 
     def __init__(self, path, create=False, options=[]):
         """Initialize the Trac environment.
@@ -189,11 +189,14 @@ class Environment(Component, ComponentManager):
         self.setup_log()
 
         from trac import core, __version__ as VERSION
+        trac_version = get_pkginfo(core).get('version', VERSION)
         self.systeminfo = [
-            ('Trac', get_pkginfo(core).get('version', VERSION)),
+            ('Trac', trac_version),
             ('Python', sys.version),
             ('setuptools', setuptools.__version__),
             ]
+        self.log.info('-' * 32 + ' environment startup [Trac %s] ' + '-' * 32,
+                      trac_version)
         self._href = self._abs_href = None
 
         from trac.loader import load_components
@@ -226,30 +229,39 @@ class Environment(Component, ComponentManager):
         This is called by the `ComponentManager` base class when a component is
         about to be activated. If this method returns false, the component does
         not get activated."""
+        if not hasattr(self, '_rules'):
+            self._rules = {}
+            for name, value in self.config.options('components'):
+                if name.endswith('.*'):
+                    name = name[:-2]
+                self._rules[name.lower()] = value.lower() in ('enabled', 'on')
+
         if not isinstance(cls, basestring):
             component_name = (cls.__module__ + '.' + cls.__name__).lower()
         else:
             component_name = cls.lower()
 
-        rules = [(name.lower(), value.lower() in ('enabled', 'on'))
-                 for name, value in self.config.options('components')]
-        rules.sort(lambda a, b: -cmp(len(a[0]), len(b[0])))
-
-        for pattern, enabled in rules:
-            if component_name == pattern or pattern.endswith('*') \
-                    and component_name.startswith(pattern[:-1]):
-                # Disable the pre-0.11 WebAdmin plugin
-                # Please note that there's no recommendation to uninstall the
-                # plugin because doing so would obviously break the backwards
-                # compatibility that the new integration administration
-                # interface tries to provide for old WebAdmin extensions
-                if component_name.startswith('webadmin.'):
-                    self.log.info('The legacy TracWebAdmin plugin has been '
-                                  'automatically disabled, and the integrated '
-                                  'administration interface will be used '
-                                  'instead.')
-                    return False
+        # Disable the pre-0.11 WebAdmin plugin
+        # Please note that there's no recommendation to uninstall the
+        # plugin because doing so would obviously break the backwards
+        # compatibility that the new integration administration
+        # interface tries to provide for old WebAdmin extensions
+        if component_name.startswith('webadmin.'):
+            self.log.info('The legacy TracWebAdmin plugin has been '
+                          'automatically disabled, and the integrated '
+                          'administration interface will be used '
+                          'instead.')
+            return False
+        
+        cname = component_name
+        while cname:
+            enabled = self._rules.get(cname)
+            if enabled is not None:
                 return enabled
+            idx = cname.rfind('.')
+            if idx < 0:
+                break
+            cname = cname[:idx]
 
         # versioncontrol components are enabled if the repository is configured
         # FIXME: this shouldn't be hardcoded like this
@@ -299,12 +311,6 @@ class Environment(Component, ComponentManager):
         If options contains ('inherit', 'file'), default values will not be
         loaded; they are expected to be provided by that file or other options.
         """
-        def _create_file(fname, data=None):
-            fd = open(fname, 'w')
-            if data:
-                fd.write(data)
-            fd.close()
-
         # Create the directory structure
         if not os.path.exists(self.path):
             os.mkdir(self.path)
@@ -313,15 +319,16 @@ class Environment(Component, ComponentManager):
         os.mkdir(os.path.join(self.path, 'plugins'))
 
         # Create a few files
-        _create_file(os.path.join(self.path, 'VERSION'),
-                     'Trac Environment Version 1\n')
-        _create_file(os.path.join(self.path, 'README'),
-                     'This directory contains a Trac environment.\n'
-                     'Visit http://trac.edgewall.org/ for more information.\n')
+        create_file(os.path.join(self.path, 'VERSION'),
+                    'Trac Environment Version 1\n')
+        create_file(os.path.join(self.path, 'README'),
+                    'This directory contains a Trac environment.\n'
+                    'Visit http://trac.edgewall.org/ for more information.\n')
 
         # Setup the default configuration
         os.mkdir(os.path.join(self.path, 'conf'))
-        _create_file(os.path.join(self.path, 'conf', 'trac.ini'))
+        create_file(os.path.join(self.path, 'conf', 'trac.ini'))
+        create_file(os.path.join(self.path, 'conf', 'trac.ini.sample'))
         skip_defaults = options and ('inherit', 'file') in [(section, option) \
                 for (section, option, value) in options]
         self.setup_config(load_defaults=not skip_defaults)
@@ -502,6 +509,8 @@ class EnvironmentSetup(Component):
             return False
         elif dbver > db_default.db_version:
             raise TracError(_('Database newer than Trac version'))
+        self.log.info("Trac database schema version is %d, should be %d",
+                      dbver, db_default.db_version)
         return True
 
     def upgrade_environment(self, db):
@@ -526,6 +535,8 @@ class EnvironmentSetup(Component):
 
     def _update_sample_config(self):
         filename = os.path.join(self.env.path, 'conf', 'trac.ini.sample')
+        if not os.path.isfile(filename):
+            return
         config = Configuration(filename)
         for section, default_options in config.defaults().iteritems():
             for name, value in default_options.iteritems():
