@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2008 Edgewall Software
+# Copyright (C) 2005-2009 Edgewall Software
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -23,6 +23,7 @@ import os
 import sys
 
 from trac.util.compat import set
+from trac.util.text import exception_to_unicode
 
 __all__ = ['load_components']
 
@@ -35,6 +36,7 @@ def _enable_plugin(env, module):
 def load_eggs(entry_point_name):
     """Loader that loads any eggs on the search path and `sys.path`."""
     def _load_eggs(env, search_path, auto_enable=None):
+        # Note that the following doesn't seem to support unicode search_path
         distributions, errors = working_set.find_plugins(
             pkg_resources.Environment(search_path)
         )
@@ -43,17 +45,18 @@ def load_eggs(entry_point_name):
             working_set.add(dist)
 
         def _log_error(item, e):
+            ue = exception_to_unicode(e)
             if isinstance(e, DistributionNotFound):
-                env.log.debug('Skipping "%s": ("%s" not found)', item, e)
+                env.log.debug('Skipping "%s": ("%s" not found)', item, ue)
             elif isinstance(e, VersionConflict):
                 env.log.error('Skipping "%s": (version conflict "%s")',
-                              item, e)
+                              item, ue)
             elif isinstance(e, UnknownExtra):
-                env.log.error('Skipping "%s": (unknown extra "%s")', item, e)
+                env.log.error('Skipping "%s": (unknown extra "%s")', item, ue)
             elif isinstance(e, ImportError):
-                env.log.error('Skipping "%s": (can\'t import "%s")', item, e)
+                env.log.error('Skipping "%s": (can\'t import "%s")', item, ue)
             else:
-                env.log.error('Skipping "%s": (error "%s")', item, e)
+                env.log.error('Skipping "%s": (error "%s")', item, ue)
 
         for dist, e in errors.iteritems():
             _log_error(dist, e)
@@ -89,8 +92,9 @@ def load_py_files():
                     if path == auto_enable:
                         _enable_plugin(env, plugin_name)
                 except Exception, e:
-                    env.log.error('Failed to load plugin from %s', plugin_file,
-                                  exc_info=True)
+                    env.log.error('Failed to load plugin from %s: %s',
+                                  plugin_file,
+                                  exception_to_unicode(e, traceback=True))
 
     return _load_py_files
 
