@@ -306,6 +306,7 @@ class Context(object):
 # Some common MIME types and their associated keywords and/or file extensions
 
 KNOWN_MIME_TYPES = {
+    'application/javascript': ['js'],
     'application/pdf':        ['pdf'],
     'application/postscript': ['ps'],
     'application/rtf':        ['rtf'],
@@ -339,7 +340,6 @@ KNOWN_MIME_TYPES = {
     'text/x-fortran':         ['f'],
     'text/x-haskell':         ['hs'],
     'text/x-ini':             ['ini', 'cfg'],
-    'text/x-javascript':      ['js'],
     'text/x-objc':            ['m', 'mm'],
     'text/x-ocaml':           ['ml', 'mli'],
     'text/x-makefile':        ['make', 'mk',
@@ -572,6 +572,25 @@ class IContentConverter(Interface):
         output_mime_type) or None if conversion is not possible."""
 
 
+class Content(object):
+    """A lazy file-like object that only reads `input` if necessary."""
+    def __init__(self, input, max_size):
+        self.input = input
+        self.max_size = max_size
+        self.content = None
+    
+    def read(self, size=-1):
+        if size == 0:
+            return ''
+        if self.content is None:
+            self.content = StringIO(self.input.read(self.max_size))
+        return self.content.read(size)
+    
+    def reset(self):
+        if self.content is not None:
+            self.content.seek(0)
+
+
 class Mimeview(Component):
     """A generic class to prettify data, typically source code."""
 
@@ -698,6 +717,10 @@ class Mimeview(Component):
             if qr > 0:
                 candidates.append((qr, renderer))
         candidates.sort(lambda x,y: cmp(y[0], x[0]))
+        
+        # Wrap file-like object so that it can be read multiple times
+        if hasattr(content, 'read'):
+            content = Content(content, self.max_preview_size)
 
         # First candidate which renders successfully wins.
         # Also, we don't want to expand tabs more than once.
@@ -706,6 +729,8 @@ class Mimeview(Component):
         for qr, renderer in candidates:
             if force_source and not getattr(renderer, 'returns_source', False):
                 continue # skip non-source renderers in force_source mode
+            if isinstance(content, Content):
+                content.reset()
             try:
                 ann_names = annotations and ', '.join(annotations) or \
                            'No annotations'
@@ -847,7 +872,7 @@ class Mimeview(Component):
         """
         # Extend default extension to MIME type mappings with configured ones
         if not self._mime_map:
-            self._mime_map = MIME_MAP
+            self._mime_map = MIME_MAP.copy()
             for mapping in self.config['mimeviewer'].getlist('mime_map'):
                 if ':' in mapping:
                     assocations = mapping.split(':')
@@ -862,6 +887,18 @@ class Mimeview(Component):
             mimetype += '; charset=' + charset
         return mimetype
 
+    def is_binary(self, mimetype=None, filename=None, content=None):
+        """Check if a file must be considered as binary."""
+        if not mimetype and filename:
+            mimetype = self.get_mimetype(filename, content)
+        if mimetype:
+            mimetype = ct_mimetype(mimetype)
+            if mimetype in self.treat_as_binary:
+                return True
+        if content is not None and is_binary(content):
+            return True
+        return False
+
     def to_utf8(self, content, mimetype=None):
         """Convert an encoded `content` to utf-8.
 
@@ -922,6 +959,8 @@ class Mimeview(Component):
         from trac.web import RequestDone
         content, output_type, ext = self.convert_content(req, in_type,
                                                          content, selector)
+        if isinstance(content, unicode):
+            content = content.encode('utf-8')
         req.send_response(200)
         req.send_header('Content-Type', output_type)
         req.send_header('Content-Length', len(content))
