@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2008 Edgewall Software
+# Copyright (C) 2006-2009 Edgewall Software
 # Copyright (C) 2006 Matthew Good <matt@matt-good.net>
 # All rights reserved.
 #
@@ -23,6 +23,7 @@ from trac.mimeview.api import IHTMLPreviewRenderer, Mimeview
 from trac.prefs import IPreferencePanelProvider
 from trac.util import get_module_path, get_pkginfo
 from trac.util.datefmt import http_date, localtz
+from trac.util.translation import _
 from trac.web import IRequestHandler
 from trac.web.chrome import add_stylesheet
 
@@ -108,9 +109,10 @@ class PygmentsRenderer(Component):
         add_stylesheet(req, '/pygments/%s.css' %
                        req.session.get('pygments_style', self.default_style))
         try:
-            mimetype = mimetype.split(';', 1)[0]
-            language = self._types[mimetype][0]
-            return self._generate(language, content)
+            if len(content) > 0:
+                mimetype = mimetype.split(';', 1)[0]
+                language = self._types[mimetype][0]
+                return self._generate(language, content)
         except (KeyError, ValueError):
             raise Exception("No Pygments lexer found for mime-type '%s'."
                             % mimetype)
@@ -118,7 +120,7 @@ class PygmentsRenderer(Component):
     # IPreferencePanelProvider implementation
 
     def get_preference_panels(self, req):
-        yield ('pygments', 'Pygments Theme')
+        yield ('pygments', _('Pygments Theme'))
 
     def render_preference_panel(self, req, panel):
         styles = list(get_all_styles())
@@ -176,9 +178,10 @@ class PygmentsRenderer(Component):
 
     def _init_types(self):
         self._types = {}
-        for _, aliases, _, mimetypes in get_all_lexers():
+        for lexname, aliases, _, mimetypes in get_all_lexers():
+            name = aliases and aliases[0] or lexname
             for mimetype in mimetypes:
-                self._types[mimetype] = (aliases[0], self.QUALITY_RATIO)
+                self._types[mimetype] = (name, self.QUALITY_RATIO)
         self._types.update(
             Mimeview(self.env).configured_modes_mapping('pygments')
         )
