@@ -36,6 +36,9 @@ __all__ = ['Ticket', 'Type', 'Status', 'Resolution', 'Priority', 'Severity',
 
 class Ticket(object):
 
+    # Fields that must not be modified directly by the user
+    protected_fields = ('resolution', 'status')
+
     id_is_valid = staticmethod(lambda num: 0 < int(num) <= 1L << 31)
 
     def __init__(self, env, tkt_id=None, db=None, version=None):
@@ -64,7 +67,7 @@ class Ticket(object):
     def _init_defaults(self, db=None):
         for field in self.fields:
             default = None
-            if field['name'] in ['resolution', 'status']:
+            if field['name'] in self.protected_fields:
                 # Ignore for new - only change through workflow
                 pass
             elif not field.get('custom'):
@@ -566,6 +569,7 @@ class Component(object):
         cursor.execute("INSERT INTO component (name,owner,description) "
                        "VALUES (%s,%s,%s)",
                        (self.name, self.owner, self.description))
+        self._old_name = self.name
 
         if handle_ta:
             db.commit()
@@ -673,6 +677,7 @@ class Milestone(object):
             ticket['milestone'] = retarget_to
             ticket.save_changes(author, 'Milestone %s deleted' % self.name,
                                 now, db=db)
+        self.name = self._old_name = None
 
         if handle_ta:
             db.commit()
@@ -693,6 +698,7 @@ class Milestone(object):
                        "VALUES (%s,%s,%s,%s)",
                        (self.name, to_timestamp(self.due), to_timestamp(self.completed),
                         self.description))
+        self._old_name = self.name
 
         if handle_ta:
             db.commit()
@@ -819,6 +825,7 @@ class Version(object):
         cursor.execute("INSERT INTO version (name,time,description) "
                        "VALUES (%s,%s,%s)",
                        (self.name, to_timestamp(self.time), self.description))
+        self._old_name = self.name
 
         if handle_ta:
             db.commit()
