@@ -39,6 +39,14 @@ from trac.web.chrome import add_ctxtnav, add_link, add_notice, \
 from trac.wiki import IWikiSyntaxProvider, WikiParser
 
 
+def cell_value(v):
+    """Normalize a cell value for display.
+    >>> (cell_value(None), cell_value(0), cell_value(1), cell_value('v'))
+    ('', '0', u'1', u'v')
+    """
+    return v is 0 and '0' or v and unicode(v) or ''
+
+
 class ReportModule(Component):
 
     implements(INavigationContributor, IPermissionRequestor, IRequestHandler,
@@ -400,6 +408,7 @@ class ReportModule(Component):
         #  - group rows according to __group__ value, if defined
         #  - group cells the same way headers are grouped
         row_groups = []
+        authorized_results = [] 
         prev_group_value = None
         for row_idx, result in enumerate(results):
             col_idx = 0
@@ -410,7 +419,7 @@ class ReportModule(Component):
             for header_group in header_groups:
                 cell_group = []
                 for header in header_group:
-                    value = unicode(result[col_idx] or '')
+                    value = cell_value(result[col_idx])
                     cell = {'value': value, 'header': header, 'index': col_idx}
                     col = header['col']
                     col_idx += 1
@@ -439,6 +448,7 @@ class ReportModule(Component):
             # FIXME: for now, we still need to hardcode the realm in the action
             if resource.realm.upper()+'_VIEW' not in req.perm(resource):
                 continue
+            authorized_results.append(result)
             if email_cells:
                 for cell in email_cells:
                     emails = Chrome(self.env).format_emails(context(resource),
@@ -474,11 +484,11 @@ class ReportModule(Component):
             return 'report.rss', data, 'application/rss+xml'
         elif format == 'csv':
             filename = id and 'report_%s.csv' % id or 'report.csv'
-            self._send_csv(req, cols, results, mimetype='text/csv',
+            self._send_csv(req, cols, authorized_results, mimetype='text/csv',
                            filename=filename)
         elif format == 'tab':
             filename = id and 'report_%s.tsv' % id or 'report.tsv'
-            self._send_csv(req, cols, results, '\t',
+            self._send_csv(req, cols, authorized_results, '\t',
                            mimetype='text/tab-separated-values',
                            filename=filename)
         else:
@@ -648,9 +658,6 @@ class ReportModule(Component):
         def iso_datetime(dt):
             return format_datetime(dt, 'iso8601')
 
-        def string(value):
-            return unicode(value or '')
-        
         col_conversions = {
             'time': iso_time,
             'datetime': iso_datetime,
@@ -660,7 +667,8 @@ class ReportModule(Component):
             'modified': iso_datetime,
         }
 
-        converters = [col_conversions.get(c.strip('_'), string) for c in cols]
+        converters = [col_conversions.get(c.strip('_'), cell_value)
+                      for c in cols]
 
         out = StringIO()
         writer = csv.writer(out, delimiter=sep)
@@ -685,10 +693,11 @@ class ReportModule(Component):
         req.perm.require('REPORT_SQL_VIEW')
 
         out = StringIO()
-        out.write('-- ## %s: %s ## --\n\n' % (id, title))
+        out.write('-- ## %s: %s ## --\n\n' % (id, title.encode('utf-8')))
         if description:
-            out.write('-- %s\n\n' % '\n-- '.join(description.splitlines()))
-        out.write(sql)
+            lines = description.encode('utf-8').splitlines()
+            out.write('-- %s\n\n' % '\n-- '.join(lines))
+        out.write(sql.encode('utf-8'))
         data = out.getvalue()
 
         req.send_response(200)
