@@ -37,7 +37,7 @@ from trac.util.datefmt import parse_date, utc, to_timestamp, to_datetime, \
 from trac.util.text import shorten_line, CRLF, to_unicode
 from trac.util.translation import _
 from trac.ticket import Milestone, Ticket, TicketSystem, group_milestones
-from trac.ticket.query import Query
+from trac.ticket.query import Query, QueryModule
 from trac.timeline.api import ITimelineEventProvider
 from trac.web import IRequestHandler, RequestDone
 from trac.web.chrome import add_link, add_notice, add_stylesheet, \
@@ -195,7 +195,8 @@ class DefaultTicketGroupStatsProvider(Component):
         for s in all_statuses:
             status_cnt[s] = 0
         if total_cnt:
-            cursor = self.env.get_db_cnx().cursor()
+            db = self.env.get_db_cnx()
+            cursor = db.cursor()
             str_ids = [str(x) for x in sorted(ticket_ids)]
             cursor.execute("SELECT status, count(status) FROM ticket "
                            "WHERE id IN (%s) GROUP BY status" %
@@ -278,8 +279,12 @@ def apply_ticket_permissions(env, req, tickets):
     return [t for t in tickets
             if 'TICKET_VIEW' in req.perm('ticket', t['id'])]
 
-def milestone_stats_data(req, stat, name, grouped_by='component', group=None):
+def milestone_stats_data(env, req, stat, name, grouped_by='component',
+                         group=None):
+    has_query = env[QueryModule] is not None
     def query_href(extra_args):
+        if not has_query:
+            return None
         args = {'milestone': name, grouped_by: group, 'group': 'status'}
         args.update(extra_args)
         return req.href.query(args)
@@ -339,7 +344,8 @@ class RoadmapModule(Component):
                                                 'owner')
             tickets = apply_ticket_permissions(self.env, req, tickets)
             stat = get_ticket_stats(self.stats_provider, tickets)
-            stats.append(milestone_stats_data(req, stat, milestone.name))
+            stats.append(milestone_stats_data(self.env, req, stat,
+                                              milestone.name))
             #milestone['tickets'] = tickets # for the iCalendar view
 
         if req.args.get('format') == 'ics':
@@ -606,7 +612,6 @@ class MilestoneModule(Component):
         old_name = milestone.name
         new_name = req.args.get('name')
         
-        milestone.name = new_name
         milestone.description = req.args.get('description', '')
 
         due = req.args.get('duedate', '')
@@ -634,7 +639,7 @@ class MilestoneModule(Component):
                     warn(_('Milestone "%(name)s" already exists, please '
                            'choose another name', name=new_name))
                 except ResourceNotFound:
-                    pass
+                    milestone.name = new_name
         else:
             warn(_('You must provide a name for the milestone.'))
 
@@ -737,7 +742,7 @@ class MilestoneModule(Component):
             'grouped_by': by,
             'groups': milestone_groups
             }
-        data.update(milestone_stats_data(req, stat, milestone.name))
+        data.update(milestone_stats_data(self.env, req, stat, milestone.name))
 
         if by:
             groups = []
@@ -766,8 +771,8 @@ class MilestoneModule(Component):
                 group_stats.append(gstat) 
 
                 gs_dict = {'name': group}
-                gs_dict.update(milestone_stats_data(req, gstat, milestone.name,
-                                                    by, group))
+                gs_dict.update(milestone_stats_data(self.env, req, gstat,
+                                                    milestone.name, by, group))
                 milestone_groups.append(gs_dict)
 
             for idx, gstat in enumerate(group_stats):
