@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2008 Edgewall Software
+# Copyright (C) 2003-2009 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
@@ -34,13 +34,14 @@ from trac.resource import Resource, get_resource_url, \
 from trac.search import ISearchSource, search_to_sql, shorten_result
 from trac.ticket.api import TicketSystem, ITicketManipulator, \
                             ITicketActionController
-from trac.ticket.model import Milestone, Ticket
+from trac.ticket.model import Milestone, Ticket, group_milestones
 from trac.ticket.notification import TicketNotifyEmail
 from trac.timeline.api import ITimelineEventProvider
-from trac.util import get_reporter_id, partition
-from trac.util.compat import any
+from trac.util import get_reporter_id
+from trac.util.compat import any, set
 from trac.util.datefmt import to_timestamp, utc
-from trac.util.text import CRLF, shorten_line, obfuscate_email_address
+from trac.util.text import CRLF, shorten_line, obfuscate_email_address, \
+                           exception_to_unicode
 from trac.util.presentation import separated
 from trac.util.translation import _
 from trac.versioncontrol.diff import get_diff_options, diff_blocks
@@ -48,7 +49,7 @@ from trac.web import IRequestHandler
 from trac.web.chrome import add_link, add_script, add_stylesheet, \
                             add_warning, add_ctxtnav, prevnext_nav, Chrome, \
                             INavigationContributor, ITemplateProvider
-from trac.wiki.formatter import format_to
+from trac.wiki.formatter import format_to, format_to_html, format_to_oneliner
 
 class InvalidTicket(TracError):
     """Exception raised when a ticket fails validation."""
@@ -75,23 +76,45 @@ class TicketModule(Component):
         """Default milestone for newly created tickets.""")
 
     default_component = Option('ticket', 'default_component', '',
-        """Default component for newly created tickets""")
+        """Default component for newly created tickets.""")
+
+    default_severity = Option('ticket', 'default_severity', '',
+        """Default severity for newly created tickets.""")
+
+    default_summary = Option('ticket', 'default_summary', '',
+        """Default summary (title) for newly created tickets.""")
+
+    default_description = Option('ticket', 'default_description', '',
+        """Default description for newly created tickets.""")
+
+    default_keywords = Option('ticket', 'default_keywords', '',
+        """Default keywords for newly created tickets.""")
+
+    default_owner = Option('ticket', 'default_owner', '',
+        """Default owner for newly created tickets.""")
+
+    default_cc = Option('ticket', 'default_cc', '',
+        """Default cc: list for newly created tickets.""")
 
     default_resolution = Option('ticket', 'default_resolution', 'fixed',
         """Default resolution for resolving (closing) tickets
         (''since 0.11'').""")
 
     timeline_details = BoolOption('timeline', 'ticket_show_details', 'false',
-        """Enable the display of all ticket changes in the timeline
-        (''since 0.9'').""")
+        """Enable the display of all ticket changes in the timeline, not only
+        open / close operations (''since 0.9'').""")
 
     max_description_size = IntOption('ticket', 'max_description_size', 262144,
         """Don't accept tickets with a too big description.
         (''since 0.11'').""")
 
+    max_comment_size = IntOption('ticket', 'max_comment_size', 262144,
+        """Don't accept tickets with a too big comment.
+        (''since 0.11.2'')""")
+
     timeline_newticket_formatter = Option('timeline', 'newticket_formatter',
                                           'oneliner',
-        """Which formatter flavor (e.g. 'default' or 'oneliner') should be
+        """Which formatter flavor (e.g. 'html' or 'oneliner') should be
         used when presenting the description for new tickets.
         If 'oneliner', the [timeline] abbreviated_messages option applies.
         (''since 0.11'').""")
@@ -103,6 +126,13 @@ class TicketModule(Component):
         but keeps the old behavior for upgraded environments (i.e. 'no').
         (''since 0.11'').""")
 
+    def _must_preserve_newlines(self):
+        preserve_newlines = self.preserve_newlines
+        if preserve_newlines == 'default':
+            preserve_newlines = self.env.get_version(initial=True) >= 21 # 0.11
+        return preserve_newlines in _TRUE_VALUES
+    must_preserve_newlines = property(_must_preserve_newlines)
+
     # IContentConverter methods
 
     def get_supported_conversions(self):
@@ -138,7 +168,7 @@ class TicketModule(Component):
     # IRequestHandler methods
 
     def match_request(self, req):
-        if re.match(r'/newticket/?$', req.path_info) is not None:
+        if req.path_info == "/newticket":
             return True
         match = re.match(r'/ticket/([0-9]+)$', req.path_info)
         if match:
@@ -147,7 +177,7 @@ class TicketModule(Component):
 
     def process_request(self, req):
         if 'id' in req.args:
-            if req.path_info.startswith('/newticket'):
+            if req.path_info == '/newticket':
                 raise TracError(_("id can't be set for a new ticket request."))
             return self._process_ticket_request(req)
         return self._process_newticket_request(req)
@@ -171,17 +201,23 @@ class TicketModule(Component):
             return
         ticket_realm = Resource('ticket')
         db = self.env.get_db_cnx()
-        sql, args = search_to_sql(db, ['b.newvalue'], terms)
-        sql2, args2 = search_to_sql(db, ['summary', 'keywords', 'description',
+        sql, args = search_to_sql(db, ['summary', 'keywords', 'description',
                                          'reporter', 'cc', 
                                          db.cast('id', 'text')], terms)
+        sql2, args2 = search_to_sql(db, ['newvalue'], terms)
+        sql3, args3 = search_to_sql(db, ['value'], terms)
         cursor = db.cursor()
-        cursor.execute("SELECT DISTINCT a.summary,a.description,a.reporter, "
-                       "a.type,a.id,a.time,a.status,a.resolution "
-                       "FROM ticket a "
-                       "LEFT JOIN ticket_change b ON a.id = b.ticket "
-                       "WHERE (b.field='comment' AND %s ) OR %s" % (sql, sql2),
-                       args + args2)
+        cursor.execute("SELECT summary,description,reporter, "
+                       "       type,id,time,status,resolution "
+                       "FROM ticket "
+                       "WHERE id IN ("
+                       "    SELECT id FROM ticket WHERE %s "
+                       "  UNION "
+                       "    SELECT ticket FROM ticket_change "
+                       "    WHERE field='comment' AND %s "
+                       "  UNION "
+                       "    SELECT ticket FROM ticket_custom WHERE %s)" %
+                       (sql, sql2, sql3), args + args2 + args3)
         ticketsystem = TicketSystem(self.env)
         for summary, desc, author, type, tid, ts, status, resolution in cursor:
             t = ticket_realm(id=tid)
@@ -194,14 +230,19 @@ class TicketModule(Component):
                                                        resolution, type)),
                        datetime.fromtimestamp(ts, utc), author,
                        shorten_result(desc, terms))
+        
+        # Attachments
+        for result in AttachmentModule(self.env).get_search_results(
+            req, ticket_realm, terms):
+            yield result        
 
     # ITimelineEventProvider methods
 
     def get_timeline_filters(self, req):
         if 'TICKET_VIEW' in req.perm:
-            yield ('ticket', _('Ticket changes'))
+            yield ('ticket', _('Opened and closed tickets'))
             if self.timeline_details:
-                yield ('ticket_details', _('Ticket details'), False)
+                yield ('ticket_details', _('Ticket updates'), False)
 
     def get_timeline_events(self, req, start, stop, filters):
         ts_start = to_timestamp(start)
@@ -310,17 +351,17 @@ class TicketModule(Component):
             descr = message = ''
             if status == 'new':
                 message = description
-                flavor = self.timeline_newticket_formatter
             else:
                 descr = info
                 message = comment
-                flavor = 'oneliner'
-            if message:
-                if self.config['timeline'].getbool('abbreviated_messages'):
-                    message = shorten_line(message)
-                descr += format_to(self.env, flavor, context(resource=ticket),
-                                   message)
-            return descr
+            t_context = context(resource=ticket)
+            t_context.set_hints(preserve_newlines=self.must_preserve_newlines)
+            if status == 'new' and \
+                    context.get_hint('wiki_flavor') == 'oneliner': 
+                flavor = self.timeline_newticket_formatter
+                t_context.set_hints(wiki_flavor=flavor,
+                                    shorten_lines=flavor == 'oneliner')
+            return descr + format_to(self.env, None, t_context, message)
 
     # Internal methods
 
@@ -346,6 +387,7 @@ class TicketModule(Component):
                 del req.args['field_owner']
 
         self._populate(req, ticket, plain_fields)
+        ticket.values['status'] = 'new'     # Force initial status
         reporter_id = req.args.get(field_reporter) or \
                       get_reporter_id(req, 'author')
         ticket.values['reporter'] = reporter_id
@@ -452,14 +494,14 @@ class TicketModule(Component):
             if problems:
                 for problem in problems:
                     add_warning(req, problem)
-                    add_warning(req,
-                                tag(tag.p('Please review your configuration, '
-                                          'probably starting with'),
-                                    tag.pre('[trac]\nworkflow = ...\n'),
-                                    tag.p('in your ', tag.tt('trac.ini'), '.'))
-                                )
+                add_warning(req,
+                            tag(tag.p('Please review your configuration, '
+                                      'probably starting with'),
+                                tag.pre('[trac]\nworkflow = ...\n'),
+                                tag.p('in your ', tag.tt('trac.ini'), '.')))
 
-            self._apply_ticket_changes(ticket, field_changes) # Apply changes made by the workflow
+            # Apply changes made by the workflow
+            self._apply_ticket_changes(ticket, field_changes)
             # Unconditionally run the validation so that the user gets
             # information any and all problems.  But it's only valid if it
             # validates and there were no problems with the workflow side of
@@ -496,9 +538,10 @@ class TicketModule(Component):
         mime = Mimeview(self.env)
         format = req.args.get('format')
         if format:
-            # FIXME: mime.send_converted(context, ticket, 'ticket_x')
+            # FIXME: mime.send_converted(context, ticket, 'ticket_x') (#3332)
+            filename = ('t%d' % ticket.id, None)[format == 'rss']
             mime.send_converted(req, 'trac.ticket.Ticket', ticket,
-                                format, 'ticket_%d' % ticket.id)
+                                format, filename=filename)
 
         def add_ticket_link(css_class, id):
             t = ticket.resource(id=id, version=None)
@@ -524,15 +567,23 @@ class TicketModule(Component):
         if global_sequence:
             db = self.env.get_db_cnx()
             cursor = db.cursor()
-            cursor.execute("SELECT max(id) FROM ticket")
-            for max_id, in cursor:
+            cursor.execute("SELECT min(id), max(id) FROM ticket")
+            for (min_id, max_id) in cursor:
+                min_id = int(min_id)
                 max_id = int(max_id)
-                if ticket.id > 1:
-                    add_ticket_link('first', 1)
-                    add_ticket_link('prev', ticket.id - 1)
+                if min_id < ticket.id:
+                    add_ticket_link('first', min_id)
+                    cursor.execute("SELECT max(id) FROM ticket WHERE id < %s",
+                                   (ticket.id,))
+                    for (prev_id,) in cursor:
+                        add_ticket_link('prev', int(prev_id))
                 if ticket.id < max_id:
-                    add_ticket_link('next', ticket.id + 1)
                     add_ticket_link('last', max_id)
+                    cursor.execute("SELECT min(id) FROM ticket WHERE %s < id",
+                                   (ticket.id,))
+                    for (next_id,) in cursor:
+                        add_ticket_link('next', int(next_id))
+                break
 
         add_stylesheet(req, 'common/css/ticket.css')
 
@@ -549,14 +600,10 @@ class TicketModule(Component):
         return 'ticket.html', data, None
 
     def _prepare_data(self, req, ticket, absurls=False):
-        preserve_newlines = self.preserve_newlines
-        if preserve_newlines == 'default':
-            preserve_newlines = self.env.get_version(initial=True) >= 21 # 0.11
-        preserve_newlines = preserve_newlines in _TRUE_VALUES
         return {'ticket': ticket,
                 'context': Context.from_request(req, ticket.resource,
                                                 absurls=absurls),
-                'preserve_newlines': preserve_newlines}
+                'preserve_newlines': self.must_preserve_newlines}
 
     def _toggle_cc(self, req, cc):
         """Return an (action, recipient) tuple corresponding to a change
@@ -590,10 +637,16 @@ class TicketModule(Component):
         return (action, entry, cc_list)
         
     def _populate(self, req, ticket, plain_fields=False):
-        fields = req.args
         if not plain_fields:
-            fields = dict([(k[6:],v) for k,v in fields.items()
+            fields = dict([(k[6:], v) for k, v in req.args.iteritems()
                            if k.startswith('field_')])
+        else:
+            fields = req.args.copy()
+        # Prevent direct changes to protected fields (status and resolution are
+        # set in the workflow, in get_ticket_changes())
+        for each in Ticket.protected_fields:
+            fields.pop(each, None)
+            fields.pop('checkbox_' + each, None)    # See Ticket.populate()
         ticket.populate(fields)
         # special case for updating the Cc: field
         if 'cc_update' in req.args:
@@ -844,21 +897,28 @@ class TicketModule(Component):
         valid = True
         resource = ticket.resource
 
-        # If the ticket has been changed, check the proper permission
+        # If the ticket has been changed, check the proper permissions
         if ticket.exists and ticket._old:
-            if 'TICKET_CHGPROP' not in req.perm(resource):
-                add_warning(req, _("No permission to change ticket fields."))
-                ticket.values.update(ticket._old)
+            # Status and resolution can be modified by the workflow even
+            # without having TICKET_CHGPROP
+            changed = set(ticket._old) - set(['status', 'resolution'])
+            if 'description' in changed \
+                    and 'TICKET_EDIT_DESCRIPTION' not in req.perm(resource):
+                add_warning(req, _("No permission to edit the ticket "
+                                   "description."))
                 valid = False
-            else: # TODO: field based checking
-                if ('description' in ticket._old and \
-                       'TICKET_EDIT_DESCRIPTION' not in req.perm(resource)) or \
-                   ('reporter' in ticket._old and \
-                       'TICKET_ADMIN' not in req.perm(resource)):
-                    add_warning(req, _("No permissions to change ticket "
-                                       "fields."))
-                    ticket.values.update(ticket._old)
-                    valid = False
+            changed.discard('description')
+            if 'reporter' in changed \
+                    and 'TICKET_ADMIN' not in req.perm(resource):
+                add_warning(req, _("No permission to change the ticket "
+                                   "reporter."))
+                valid = False
+            changed.discard('reporter')
+            if changed and 'TICKET_CHGPROP' not in req.perm(resource):
+                add_warning(req, _("No permission to change ticket fields."))
+                valid = False
+            if not valid:
+                ticket.values.update(ticket._old)
 
         comment = req.args.get('comment')
         if comment:
@@ -905,6 +965,13 @@ class TicketModule(Component):
                           num=self.max_description_size))
             valid = False
 
+        # Validate comment length
+        if len(comment or '') > self.max_comment_size:
+            add_warning(req, _('Ticket comment is too long (must be less '
+                               'than %(num)s characters)',
+                               num=self.max_comment_size))
+            valid = False
+
         # Validate comment numbering
         try:
             # comment index must be a number
@@ -931,20 +998,21 @@ class TicketModule(Component):
 
     def _do_create(self, req, ticket):
         ticket.insert()
-        req.perm(ticket.resource).require('TICKET_VIEW')
 
         # Notify
         try:
             tn = TicketNotifyEmail(self.env)
             tn.notify(ticket, newticket=True)
         except Exception, e:
-            self.log.exception("Failure sending notification on creation of "
-                               "ticket #%s: %s" % (ticket.id, e))
+            self.log.error("Failure sending notification on creation of "
+                    "ticket #%s: %s", ticket.id, exception_to_unicode(e))
 
         # Redirect the user to the newly created ticket or add attachment
         if 'attachment' in req.args:
             req.redirect(req.href.attachment('ticket', ticket.id,
                                              action='new'))
+        if 'TICKET_VIEW' not in req.perm('ticket', ticket.id):
+            req.redirect(req.href.newticket())
         req.redirect(req.href.ticket(ticket.id))
 
     def _do_save(self, req, ticket, action):
@@ -968,8 +1036,8 @@ class TicketModule(Component):
                 tn = TicketNotifyEmail(self.env)
                 tn.notify(ticket, newticket=False, modtime=now)
             except Exception, e:
-                self.log.exception("Failure sending notification on change to "
-                                   "ticket #%s: %s" % (ticket.id, e))
+                self.log.error("Failure sending notification on change to "
+                        "ticket #%s: %s", ticket.id, exception_to_unicode(e))
 
         # After saving the changes, apply the side-effects.
         for controller in controllers:
@@ -1028,6 +1096,7 @@ class TicketModule(Component):
     def _prepare_fields(self, req, ticket):
         context = Context.from_request(req, ticket.resource)
         fields = []
+        owner_field = None
         for field in ticket.fields:
             name = field['name']
             type_ = field['type']
@@ -1037,29 +1106,25 @@ class TicketModule(Component):
                         'resolution'):
                 field['skip'] = True
             elif name == 'owner':
+                TicketSystem(self.env).eventually_restrict_owner(field, ticket)
+                type_ = field['type']
                 field['skip'] = True
                 if not ticket.exists:
                     field['label'] = 'Assign to'
                     if 'TICKET_MODIFY' in req.perm(ticket.resource):
                         field['skip'] = False
+                        owner_field = field
             elif name == 'milestone':
-                milestones = [(opt, Milestone(self.env, opt))
+                milestones = [Milestone(self.env, opt)
                               for opt in field['options']]
-                milestones = [(opt, m) for opt, m in milestones
+                milestones = [m for m in milestones
                               if 'MILESTONE_VIEW' in req.perm(m.resource)]
-                open_milestones, closed_milestones = \
-                        partition([(opt, m.is_completed)
-                                   for opt, m in milestones],
-                                  (False, True))
-                if ticket.exists and \
-                       'TICKET_ADMIN' in req.perm(ticket.resource):
-                    field['options'] = []
-                    field['optgroups'] = [
-                        {'label': _('Open'), 'options': open_milestones},
-                        {'label': _('Closed'), 'options': closed_milestones},
-                    ]
-                else:
-                    field['options'] = open_milestones
+                groups = group_milestones(milestones, ticket.exists 
+                    and 'TICKET_ADMIN' in req.perm(ticket.resource))
+                field['options'] = []
+                field['optgroups'] = [
+                    {'label': label, 'options': [m.name for m in milestones]}
+                    for (label, milestones) in groups]
                 milestone = Resource('milestone', ticket[name])
                 field['rendered'] = render_resource_link(self.env, context,
                                                          milestone, 'compact')
@@ -1095,12 +1160,26 @@ class TicketModule(Component):
                 value = ticket.values.get(name)
                 if value in ('1', '0'):
                     field['rendered'] = value == '1' and _('yes') or _('no')
-                  
+            elif type_ == 'text':
+                if field.get('format') == 'wiki':
+                    field['rendered'] = format_to_oneliner(self.env, context,
+                                                           ticket[name])
+            elif type_ == 'textarea':
+                if field.get('format') == 'wiki':
+                    field['rendered'] = \
+                        format_to_html(self.env, context, ticket[name],
+                                escape_newlines=self.must_preserve_newlines)
+            
             # ensure sane defaults
             field.setdefault('optional', False)
             field.setdefault('options', [])
             field.setdefault('skip', False)
             fields.append(field)
+        
+        # Move owner field to end when shown
+        if owner_field is not None:
+            fields.remove(owner_field)
+            fields.append(owner_field)
         return fields
         
     def _insert_ticket_data(self, req, ticket, data, author_id, field_changes):
@@ -1121,7 +1200,8 @@ class TicketModule(Component):
         def quote_original(author, original, link):
             if 'comment' not in req.args: # i.e. the comment was not yet edited
                 data['comment'] = '\n'.join(
-                    ['Replying to [%s %s]:' % (link, author)] +
+                    ['Replying to [%s %s]:' % (link,
+                                        obfuscate_email_address(author))] +
                     ['> %s' % line for line in original.splitlines()] + [''])
 
         if replyto == 'description':
@@ -1281,7 +1361,7 @@ class TicketModule(Component):
                     'EMAIL_VIEW' in req.perm(resource_new or ticket.resource)):
                 render_elt = obfuscate_email_address
         elif field == 'keywords':
-            old_list, new_list = old.split(), new.split()
+            old_list, new_list = (old or '').split(), new.split()
             sep = ' '
         if (old_list, new_list) != (None, None):
             added = [tag.em(render_elt(x)) for x in new_list 
