@@ -58,6 +58,7 @@ from trac.versioncontrol import Changeset, Node, Repository, \
 from trac.versioncontrol.cache import CachedRepository
 from trac.versioncontrol.svn_authz import SubversionAuthorizer
 from trac.util import embedded_numbers
+from trac.util.compat import sorted
 from trac.util.text import exception_to_unicode, to_unicode
 from trac.util.translation import _
 from trac.util.datefmt import utc
@@ -236,15 +237,18 @@ class SubversionConnector(Component):
     implements(IRepositoryConnector)
 
     branches = ListOption('svn', 'branches', 'trunk,branches/*', doc=
-        """List of paths categorized as ''branches''.
-        If a path ends with '*', then all the directory entries found
-        below that path will be included.
+        """Comma separated list of paths categorized as branches.
+        If a path ends with '*', then all the directory entries found below 
+        that path will be included. 
+        Example: `/trunk, /branches/*, /projectAlpha/trunk, /sandbox/*`
         """)
 
     tags = ListOption('svn', 'tags', 'tags/*', doc=
-        """List of paths categorized as ''tags''.
-        If a path ends with '*', then all the directory entries found
-        below that path will be included.
+        """Comma separated list of paths categorized as tags.
+        
+        If a path ends with '*', then all the directory entries found below
+        that path will be included.
+        Example: `/tags/*, /projectAlpha/tags/A-1.0, /projectAlpha/tags/A-v1.1`
         """)
 
     error = None
@@ -456,6 +460,8 @@ class SubversionRepository(Repository):
         path_utf8 = _to_svn(self.scope, path)
         if start < end:
             start, end = end, start
+        if (start, end) == (1, 0): # only happens for empty repos
+            return
         root = fs.revision_root(self.fs_ptr, start, pool())
         # fs.node_history leaks when path doesn't exist (#6588)
         if fs.check_path(root, path_utf8, pool()) == core.svn_node_none:
@@ -482,7 +488,7 @@ class SubversionRepository(Repository):
     
     def _previous_rev(self, rev, path='', pool=None):
         if rev > 1: # don't use oldest here, as it's too expensive
-            for _, prev in self._history(path, 0, rev-1, pool or self.pool):
+            for _, prev in self._history(path, 1, rev-1, pool or self.pool):
                 return prev
         return None
     
@@ -500,7 +506,7 @@ class SubversionRepository(Repository):
         if not self.youngest:
             self.youngest = fs.youngest_rev(self.fs_ptr, self.pool())
             if self.scope != '/':
-                for path, rev in self._history('', 0, self.youngest, self.pool):
+                for path, rev in self._history('', 1, self.youngest, self.pool):
                     self.youngest = rev
                     break
         return self.youngest
@@ -557,7 +563,7 @@ class SubversionRepository(Repository):
                     yield path, rev+1, Changeset.DELETE
                 newer = None # 'newer' is the previously seen history tuple
                 older = None # 'older' is the currently examined history tuple
-                for p, r in self._history(path, 0, rev, subpool):
+                for p, r in self._history(path, 1, rev, subpool):
                     older = (_path_within_scope(self.scope, p), r,
                              Changeset.ADD)
                     rev = self._previous_rev(r, pool=subpool)
@@ -713,7 +719,7 @@ class SubversionNode(Node):
         older = None # 'older' is the currently examined history tuple
         pool = Pool(self.pool)
         numrevs = 0
-        for path, rev in self.repos._history(self.path, 0, self._requested_rev,
+        for path, rev in self.repos._history(self.path, 1, self._requested_rev,
                                              pool):
             path = _path_within_scope(self.scope, path)
             if rev > 0 and path:
@@ -729,7 +735,7 @@ class SubversionNode(Node):
                 newer = older
             if limit and numrevs >= limit:
                 break
-        if newer:
+        if newer and (not limit or numrevs < limit):
             yield newer
 
     def get_annotations(self):
@@ -738,7 +744,7 @@ class SubversionNode(Node):
             def blame_receiver(line_no, revision, author, date, line, pool):
                 annotations.append(revision)
             try:
-                rev = _svn_rev(self.rev)
+                rev = _svn_rev(self._requested_rev)
                 start = _svn_rev(0)
                 file_url_utf8 = posixpath.join(self.repos.ra_url_utf8,
                                                self._scoped_path_utf8)
@@ -786,6 +792,12 @@ class SubversionNode(Node):
         return fs.node_prop(self.root, self._scoped_path_utf8, name,
                             self.pool())
 
+    def get_branch_origin(self):
+        """Return the revision in which the node's path was created"""
+        root_and_path = fs.closest_copy(self.root, self._scoped_path_utf8)
+        if root_and_path:
+            return fs.revision_root_revision(root_and_path[0])
+
     def get_copy_ancestry(self):
         """Retrieve the list of `(path,rev)` copy ancestors of this node.
         Most recent ancestor first. Each ancestor `(path, rev)` corresponds 
@@ -809,7 +821,7 @@ class SubversionNode(Node):
                     relpath = previous_path[len(path):].strip('/')
                 copied_from = fs.copied_from(root, path)
                 if copied_from:
-                    (rev, path) = fs.copied_from(root, path)
+                    (rev, path) = copied_from
                     path = path.lstrip('/')
                     root = fs.revision_root(self.fs_ptr, rev, self.pool())
                     if relpath:
