@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2008 Edgewall Software
+# Copyright (C) 2005-2009 Edgewall Software
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -26,9 +26,10 @@ import sys
 import urlparse
 
 from trac.core import Interface, TracError
-from trac.util import get_last_traceback
+from trac.util import get_last_traceback, md5, unquote
 from trac.util.datefmt import http_date, localtz
 from trac.web.href import Href
+from trac.web.wsgi import _FileWrapper
 
 HTTP_STATUS = dict([(code, reason.title()) for code, (reason, description)
                     in BaseHTTPRequestHandler.responses.items()])
@@ -98,6 +99,31 @@ class _RequestArgs(dict):
         return val
 
 
+def parse_query_string(query_string):
+    """Parse a query string into a _RequestArgs."""
+    args = _RequestArgs()
+    for arg in query_string.split('&'):
+        nv = arg.split('=', 1)
+        if len(nv) == 2:
+            (name, value) = nv
+        else:
+            (name, value) = (nv[0], '')
+        name = unquote(name.replace('+', ' '))
+        if isinstance(name, unicode):
+            name = name.encode('utf-8')
+        value = unquote(value.replace('+', ' '))
+        if not isinstance(value, unicode):
+            value = unicode(value, 'utf-8')
+        if name in args:
+            if isinstance(args[name], list):
+                args[name].append(value)
+            else:
+                args[name] = [args[name], value]
+        else:
+            args[name] = value
+    return args
+
+
 class RequestDone(Exception):
     """Marker exception that indicates whether request processing has completed
     and a response was sent.
@@ -154,6 +180,7 @@ class Request(object):
             'incookie': Request._parse_cookies,
             '_inheaders': Request._parse_headers
         }
+        self.redirect_listeners = []
 
         self.base_url = self.environ.get('trac.base_url')
         if not self.base_url:
@@ -180,6 +207,9 @@ class Request(object):
                       doc='The HTTP method of the request')
     path_info = property(fget=lambda self: self.environ.get('PATH_INFO', '').decode('utf-8'),
                          doc='Path inside the application')
+    query_string = property(fget=lambda self: self.environ.get('QUERY_STRING',
+                                                               ''),
+                            doc='Query part of the request')
     remote_addr = property(fget=lambda self: self.environ.get('REMOTE_ADDR'),
                            doc='IP address of the remote user')
     remote_user = property(fget=lambda self: self.environ.get('REMOTE_USER'),
@@ -194,6 +224,13 @@ class Request(object):
     server_port = property(fget=lambda self: int(self.environ['SERVER_PORT']),
                            doc='Port number the server is bound to')
 
+    def add_redirect_listener(self, listener):
+        """Add a callable to be called prior to executing a redirect.
+        
+        The callable is passed the arguments to the `redirect()` call.
+        """
+        self.redirect_listeners.append(listener)
+
     def get_header(self, name):
         """Return the value of the specified HTTP header, or `None` if there's
         no such header in the request.
@@ -243,8 +280,7 @@ class Request(object):
         so that consecutive requests can be cached.
         """
         if isinstance(extra, list):
-            import md5
-            m = md5.new()
+            m = md5()
             for elt in extra:
                 m.update(repr(elt))
             extra = m.hexdigest()
@@ -254,6 +290,7 @@ class Request(object):
             self.send_header('ETag', etag)
         else:
             self.send_response(304)
+            self.send_header('Content-Length', 0)
             self.end_headers()
             raise RequestDone
 
@@ -262,8 +299,10 @@ class Request(object):
         `url` may be relative or absolute, relative URLs will be translated
         appropriately.
         """
-        if self.session:
-            self.session.save() # has to be done before the redirect is sent
+        for listener in self.redirect_listeners:
+            listener(self, url, permanent)
+        
+        self.session.save() # has to be done before the redirect is sent
 
         if permanent:
             status = 301 # 'Moved Permanently'
@@ -280,13 +319,11 @@ class Request(object):
 
         self.send_header('Location', url)
         self.send_header('Content-Type', 'text/plain')
+        self.send_header('Content-Length', 0)
         self.send_header('Pragma', 'no-cache')
         self.send_header('Cache-control', 'no-cache')
         self.send_header('Expires', 'Fri, 01 Jan 1999 00:00:00 GMT')
         self.end_headers()
-
-        if self.method != 'HEAD':
-            self.write('Redirecting...')
         raise RequestDone
 
     def display(self, template, content_type='text/html', status=200):
@@ -378,6 +415,7 @@ class Request(object):
         last_modified = http_date(mtime)
         if last_modified == self.get_header('If-Modified-Since'):
             self.send_response(304)
+            self.send_header('Content-Length', 0)
             self.end_headers()
             raise RequestDone
 
@@ -392,10 +430,9 @@ class Request(object):
         self.end_headers()
 
         if self.method != 'HEAD':
-            self._response = file(path, 'rb')
-            file_wrapper = self.environ.get('wsgi.file_wrapper')
-            if file_wrapper:
-                self._response = file_wrapper(self._response, 4096)
+            fileobj = file(path, 'rb')
+            file_wrapper = self.environ.get('wsgi.file_wrapper', _FileWrapper)
+            self._response = file_wrapper(fileobj, 4096)
         raise RequestDone
 
     def read(self, size=None):
@@ -422,7 +459,12 @@ class Request(object):
             self.end_headers()
         if isinstance(data, unicode):
             data = data.encode(self._outcharset or 'utf-8')
-        self._write(data)
+        try:
+            self._write(data)
+        except (IOError, socket.error), e:
+            if e.args[0] in (errno.EPIPE, errno.ECONNRESET, 10053, 10054):
+                raise RequestDone
+            raise
 
     # Internal methods
 
@@ -440,8 +482,14 @@ class Request(object):
         if ctype not in ('application/x-www-form-urlencoded',
                          'multipart/form-data'):
             fp = StringIO('')
-
+        # Python 2.6 introduced a backwards incompatible change for
+        # FieldStorage where QUERY_STRING is no longer ignored for POST
+        # requests. We'll keep the pre 2.6 behaviour for now...
+        if self.method == 'POST':
+            qs_on_post = self.environ.pop('QUERY_STRING', '')
         fs = cgi.FieldStorage(fp, environ=self.environ, keep_blank_values=True)
+        if self.method == 'POST':
+            self.environ['QUERY_STRING'] = qs_on_post
         if fs.list:
             for name in fs.keys():
                 values = fs[name]
